﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;

namespace SharePointArchitects.FilePlanner
{
    public class SharePointList
    {
        public static void HandleException(Exception x, string fmt, params object[] args)
        {
            //LogService.LogError(string.Format("{0} --> {1}", string.Format(fmt, args), x.ToString()));
        }

        /// <summary>
        /// Creates a list of the specified type.
        /// </summary>
        /// <param name="web"></param>
        /// <param name="type"></param>
        /// <param name="title"></param>
        /// <param name="description"></param>
        /// <returns></returns>
        public static SPList Create(SPWeb web, SPListTemplateType type, string title, string description)
        {
            Guid listid = Guid.Empty;
            SPList list = null;
            try { list = web.Lists[title]; }
            catch { }
            if (list == null)
            {
                listid = web.Lists.Add(title, description, type);
                list = web.Lists[listid];
            }
            return list;
        }

        /// <summary>
        /// Creates a list and then associates it with a given set of content types.
        /// </summary>
        /// <param name="web"></param>
        /// <param name="type"></param>
        /// <param name="title"></param>
        /// <param name="description"></param>
        /// <param name="contentTypes"></param>
        /// <returns></returns>
        public static SPList Create(SPWeb web, SPListTemplateType type, string title, string description, params SPContentType[] contentTypes)
        {
            SPList list = Create(web, type, title, description);
            if (list != null)
            {
                if (contentTypes.Length > 0)
                {
                    list.ContentTypesEnabled = true;
                    foreach (SPContentType ct in contentTypes)
                    {
                        try
                        {
                            list.ContentTypes.Add(ct);
                        }
                        catch { }
                    }
                    list.Update();
                }
            }
            return list;
        }

        /// <summary>
        /// Removes the specified list (if found) from the web.
        /// </summary>
        /// <param name="web"></param>
        /// <param name="title"></param>
        /// <returns></returns>
        public static bool Delete(SPWeb web, string title)
        {
            try
            {
                SPList list = web.Lists[title];
                if (list != null)
                {
                    list.Delete();
                    return true;
                }
            }
            catch
            {
            }
            return false;
        }

        /// <summary>
        /// Searches for a given list in a specified website.
        /// </summary>
        /// <param name="web"></param>
        /// <param name="title"></param>
        /// <returns></returns>
        public static SPList Find(SPWeb web, string title)
        {
            SPList list = null;
            try
            {
                list = web.Lists[title];
            }
            catch
            {
            }
            return list;
        }

        /// <summary>
        /// Creates a folder in the list having the specified name.
        /// Returns the existing folder if already there.
        /// </summary>
        /// <param name="list"></param>
        /// <param name="folderName"></param>
        /// <returns></returns>
        public static SPFolder CreateFolder(SPList list, string folderName)
        {
            try
            {
                // search for an existing folder...
                foreach (SPFolder folder in list.RootFolder.SubFolders)
                    if (folder.Name.Equals(folderName))
                        return folder;

                // not found, so create a new one
                return list.RootFolder.SubFolders.Add(folderName);
            }
            catch (Exception x)
            {
                HandleException(x, "Exception creating folder '{0}'", folderName);
            }
            return null;
        }

        /// <summary>
        /// Creates a column in a list.
        /// </summary>
        /// <param name="list"></param>
        /// <param name="fieldName"></param>
        /// <param name="fieldType"></param>
        /// <param name="required"></param>
        /// <returns></returns>
        public static SPField CreateColumn(SPList list,
            string fieldName, SPFieldType fieldType, bool required)
        {
            SPField field = null;
            SPWeb web = list.ParentWeb;
            try
            {
                field = web.AvailableFields[fieldName];
            }
            catch { }
            if (field != null)
            {
                foreach (SPField f in list.Fields)
                    if (f.Id == field.Id)
                        return f;
            }
            try
            {
                string fName = web.Fields.Add(fieldName, fieldType, required);
                field = web.Fields[fName];
                list.Fields.Add(field);
            }
            catch (Exception x)
            {
                HandleException(x, "Failed to create field '{0}'", fieldName);
            }
            return field;
        }

        /// <summary>
        /// Adds the specified content type to the list of content
        /// types recognized by a given list.
        /// </summary>
        /// <param name="list">the SPList object to be modified</param>
        /// <param name="contentTypeName">the name of the content type to be added</param>
        public static bool AddContentType(SPList list, string contentTypeName)
        {
            try
            {
                SPContentType ct = list.ParentWeb.AvailableContentTypes[contentTypeName];
                if (ct == null)
                    throw new SPException(string.Format(
                        "Content type '{0}' not found", contentTypeName));
                list.ContentTypes.Add(ct);
                return true;
            }
            catch (SPException x)
            {
                HandleException(x, "Failed to add content type '{0}' to list '{1}'",
                    contentTypeName, list.Title);
            }
            return false;
        }

        /// <summary>
        /// Removes the specified content type from a list.
        /// </summary>
        /// <param name="list">an SPList object</param>
        /// <param name="contentTypeName">the name of the content type to be removed</param>
        public static bool RemoveContentType(SPList list, string contentTypeName)
        {
            try
            {
                SPContentType ct = list.ContentTypes[contentTypeName];
                if (ct != null)
                    list.ContentTypes.Delete(ct.Id);
                return true;
            }
            catch (SPException x)
            {
                HandleException(x, "Failed to remove content type '{0}' from list '{1}'",
                    contentTypeName, list.Title);
            }
            return false;
        }
    }

}
