﻿using System;
using System.Collections.Generic;
using System.DirectoryServices;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Security;
using System.Xml;
using System.Xml.Serialization;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.Office.RecordsManagement.RecordsRepository;
using SharePointArchitects.FilePlanner.Configuration;

namespace SharePointArchitects.FilePlanner
{
    // Extensions to the FilePlan serialization component to support
    // records center provisioning and configuration.
    public partial class FilePlan
    {
        #region Records Center Provisioning

        // Creates a record center in a new web application, and then
        // provisions and configures it.
        public SPSite Provision(FilePlanSettings settings)
        {
            SPWebApplication webApplication = null;

            // Use the SPWebApplicationBuilder to create a new web application.
            SPWebApplicationBuilder builder = new SPWebApplicationBuilder(SPFarm.Local);
            builder.Port = settings.PortNumber;
            builder.CreateNewDatabase = settings.NewDatabase;
            builder.UseNTLMExclusively = true;
            builder.AllowAnonymousAccess = false;
            builder.UseSecureSocketsLayer = settings.SSLEnabled;
            builder.ApplicationPoolId = settings.ApplicationPoolName;
            builder.ApplicationPoolUsername = settings.AdministratorLogin;
            builder.ApplicationPoolPassword = new SecureString();
            foreach (Char c in settings.AdministratorPassword.ToCharArray())
                builder.ApplicationPoolPassword.AppendChar(c);

            // Create an application with the new application pool
            webApplication = builder.Create();
            webApplication.Name = settings.Title;
            webApplication.Update();

            // Provision the content database.
            webApplication.Provision();

            // Add the new application to central admin.
            SPWebService.AdministrationService.WebApplications.Add(webApplication);

            // Recycle the app pool.
            using (DirectoryEntry pool = new DirectoryEntry(
                string.Format("IIS://localhost/w3svc/apppools/{0}", settings.ApplicationPoolName)))
            {
                pool.Invoke("Recycle");
            }

            return Provision(webApplication, settings);
        }

        // Creates a records center within a given web application, and then
        // configures it according to the specified File Plan settings.
        public SPSite Provision(SPWebApplication webApplication, FilePlanSettings settings)
        {
            Exception exception = null;
            SPSite recordsCenterSite = null;

            // specify the name of the template to use
            string template = "OFFILE#1";

            try
            {
                // Create a site collection for the new records center site.
                recordsCenterSite = webApplication.Sites.Add(settings.GetSafeUrl(), settings.Title, settings.Description,
                    1033, template, settings.AdministratorLogin, settings.AdministratorDisplayName,
                    settings.AdministratorEmail);

                Configure(recordsCenterSite, settings, out exception);
            }
            catch (Exception x)
            {
                HandleException(x, "Failed to create Records Center site collection for file plan '{0}'", 
                    settings.Title);
            }

            return recordsCenterSite;
        }

        #endregion

        #region Records Center Site Configuration

        // Configures an existing records center site according to the specified File Plan settings.
        public bool Configure(SPSite recordsCenter, FilePlanSettings settings, out Exception exception)
        {
            bool success = true;
            exception = null;
            EcmDocumentRoutingWeb routingWeb = new EcmDocumentRoutingWeb(recordsCenter.RootWeb);
            if (routingWeb == null)
            {
                exception = new Exception("RoutingWeb was null");
                return false;
            }
            return new RecordsCenterConfiguration(this,settings)
                .Configure(recordsCenter,routingWeb,out exception);
        }

        #endregion

        #region Serialization/Deserialization support

        public static FilePlan LoadFromListItem(SPListItem item)
        {
            if (item.File != null)
                return LoadFromSPFile(item.File);
            Log("Cannot load file plan.  List item '{0}' has no file.", item.Title);
            return null;
        }

        public static FilePlan LoadFromSPFile(SPFile file)
        {
            FilePlan filePlan = null;
            try
            {
                using (Stream stream = file.OpenBinaryStream())
                {
                    XmlSerializer ser = new XmlSerializer(typeof(FilePlan), Properties.Resources.FILE_PLAN_NAMESPACE);
                    return (FilePlan)ser.Deserialize(stream);
                }
            }
            catch (SPException x)
            {
                HandleException(x, "Failed to load file plan from list item.");
            }
            return filePlan;
        }

        #endregion

        #region Diagnostic routines

        public static void Log(string format, params object[] args)
        {
            Trace.WriteLine(string.Format(format, args));
        }

        public static void HandleException(Exception exception, string format, params object[] args)
        {
            Log(format, args);
            Log("Exception: {0}", exception.ToString());
        }

        #endregion
    }
}
