﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.Office.RecordsManagement;
using Microsoft.Office.RecordsManagement.InformationPolicy;

namespace SharePointArchitects.FilePlanner.Configuration
{
    /// <summary>
    /// Describes a SharePoint content type.
    /// </summary>
    public class ContentTypeDescriptor : SharePointObjectDescriptor
    {
        public string Title = string.Empty;
        public string Description = string.Empty;
        public FilePlan FilePlan = null;
        public InformationPolicyDescriptor Policy = null;
        public SPContentType ContentType = null;
        public List<RecordSpecification> AssociatedRecords = new List<RecordSpecification>();
        public List<SharePointObjectDescriptor> RoutingTargets = new List<SharePointObjectDescriptor>();

        // Creates a content type descriptor based on a record specification
        // and grouping strategy.
        public ContentTypeDescriptor(FilePlan filePlan, RecordSpecification record, RecordGroupingStrategy grouping)
        {
            FilePlan = filePlan;
            AssociatedRecords.Add(record);
            record.ContentType = this;

            // Default strategy creates one content type per record type.
            Title = record.Title.Sanitize();
            Description = record.Description;

            // Default strategy constructs a retention policy from the retention formula
            // associated with each record.
            if (record.HasRetention)
                Policy = new InformationPolicyDescriptor(typeof(ContentTypeDescriptor), record);
        }

        // Creates the content type within the specified records center site collection.
        // If the type has an associated information policy, it is attached to the type.
        public bool Configure(SPSite recordsCenter, out Exception exception)
        {
            exception = null;
            SPWeb web = recordsCenter.RootWeb;

            // Get the Document content type as the base.
            SPContentType baseType = web.AvailableContentTypes[SPBuiltInContentTypeId.Document];

            // Create a new content type having the name.
            try
            {
                this.ContentType = new SPContentType(baseType, web.ContentTypes, Title);
                web.ContentTypes.Add(this.ContentType);
                // Note: A content type is not initialized until after it is added.
                this.ContentType = web.ContentTypes[this.ContentType.Id];
                this.ContentType.Group = FilePlan.Title.Sanitize();
                this.ContentType.Description = Description;
                this.ContentType.Hidden = false;
                this.ContentType.Sealed = false;
                // *** code to add columns ***
                this.ContentType.Update();
            }
            catch (Exception x)
            {
                try
                {
                    this.ContentType = web.ContentTypes[Title];
                }
                catch (Exception y)
                {
                    exception = y;
                    return false;
                }
            }

            if (this.ContentType != null)
            {
                // Attach the information policy, if any, to the content type
                if (Policy != null)
                    return Policy.Configure(this.ContentType, out exception);
                return true;
            }

            exception = new Exception(String.Format("Failed to create content type '{0}'", Title));
            return false;
        }
    }
}
