﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.Office.RecordsManagement;
using Microsoft.Office.RecordsManagement.InformationPolicy;

namespace SharePointArchitects.FilePlanner.Configuration
{
    /// <summary>
    /// Describes a SharePoint document library.
    /// </summary>
    public class DocumentLibraryDescriptor : SharePointObjectDescriptor
    {
        public string Title = string.Empty;
        public string Description = string.Empty;
        public SPDocumentLibrary Library = null;
        public List<FolderDescriptor> Folders = new List<FolderDescriptor>();
        public List<ContentTypeDescriptor> ContentTypes = new List<ContentTypeDescriptor>();
        public List<InformationPolicyDescriptor> InformationPolicies = new List<InformationPolicyDescriptor>();

        // Creates a document library descriptor for a file plan based on a category path.
        public DocumentLibraryDescriptor(FilePlan plan, ContentTypeDescriptor contentType, string[] categories)
        {
            Title = categories[0].Trim().Sanitize();
            Description = string.Format("Generated from File Plan '{0}'", plan.Title);
            ContentTypes.Add(contentType);

            if (categories.Length > 1)
            {
                // Delegate to the folder to create its sub-folders.
                Folders.Add(new FolderDescriptor(contentType, categories[1], categories, 2));
            }
            else
            {
                contentType.RoutingTargets.Add(this);
            }
        }

        // Return the path to the root folder when used as the
        // target of a routing rule.
        public override string RoutingRuleTargetPath
        {
            get
            {
                return Library.RootFolder.ServerRelativeUrl;
            }
        }

        // Creates the document library in the root web of the site collection.
        // If the library contains folders, they are created as well.
        // If content types are associated with the library, the associations are installed.
        // If information policies are associated with the library, they are installed.
        public bool Configure(SPSite recordsCenter, out Exception exception)
        {
            exception = null;

            try
            {
                // Create the library.
                Library = SharePointList.Create(recordsCenter.RootWeb,
                    SPListTemplateType.DocumentLibrary, Title, Description) as SPDocumentLibrary;

                if (Library == null)
                {
                    exception = new Exception(string.Format("Failed to create document library '{0}'", Title));
                    return false;
                }

                // Configure general settings.
                Library.ContentTypesEnabled = (ContentTypes.Count > 0);
                Library.OnQuickLaunch = true;

                // Add the content types.
                foreach (ContentTypeDescriptor contentType in ContentTypes)
                    SharePointList.AddContentType(Library, contentType.Title);

                // Add the policies.
                foreach (InformationPolicyDescriptor policy in InformationPolicies)
                    if (!policy.Configure(Library, out exception))
                        return false;

                // Create the folders.
                foreach (FolderDescriptor folder in Folders)
                    if (!folder.Configure(Library.RootFolder, out exception))
                        return false;

                Library.Update();
            }
            catch (Exception x)
            {
                exception = x;
                return false;
            }

            return true;
        }
    }
}
