﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.Office.RecordsManagement;
using Microsoft.Office.RecordsManagement.InformationPolicy;
using Microsoft.Office.RecordsManagement.PolicyFeatures;

namespace SharePointArchitects.FilePlanner.Configuration
{
    /// <summary>
    /// Describes a SharePoint information policy.
    /// </summary>
    public class InformationPolicyDescriptor : SharePointObjectDescriptor
    {
        public string PolicyId = string.Empty;
        public string PolicyName = string.Empty;
        public string PolicyStatement = string.Empty;
        public string PolicyDescription = string.Empty;
        public string RetentionFormulaXml = string.Empty;

        // Constructs a new policy descriptor for a given retention formula.
        // The retention formula is parsed from an ECML expression that can represent
        // multi-stage retention formulas.
        public InformationPolicyDescriptor(Type sourceType, RecordSpecification record)
        {
            PolicyId = Guid.NewGuid().ToString().Replace("{", "").Replace("}", "");
            PolicyName = string.Format("UnnamedPolicy");
            PolicyStatement = string.Format("Policy generated for type '{0}'", sourceType.Name);
            PolicyDescription = string.Format("Associated record type is '{0}'", record.Title);
            RetentionFormulaXml = record.Retention.Parse();
        }

        // Retrieves the CAML policy declaration.
        public string PolicyXml
        {
            get
            {
                return Properties.Resources.InformationPolicyTemplate
                    .Replace("$PolicyId", PolicyId)
                    .Replace("$PolicyName", PolicyName)
                    .Replace("$PolicyDescription", PolicyDescription)
                    .Replace("$PolicyStatement", PolicyStatement)
                    .Replace("$PolicyItems", RetentionFormulaXml);
            }
        }

        // Configures an information policy at the site collection level.
        public bool Configure(SPSite recordsCenter, out Exception exception)
        {
            exception = null;
            try
            {
                PolicyCollection.Add(recordsCenter, PolicyXml);
            }
            catch
            {
                try
                {
                    PolicyCollection.Delete(recordsCenter, PolicyId);
                    PolicyCollection.Add(recordsCenter, PolicyXml);
                }
                catch (Exception x)
                {
                    exception = x;
                    return false;
                }
            }
            return true;
        }

        // Configures an information policy for a content type.
        public bool Configure(SPContentType contentType, out Exception exception)
        {
            exception = null;
            Policy policy = Policy.GetPolicy(contentType);
            try
            {
                if (policy == null)
                {
                    Policy.CreatePolicy(contentType, null);
                    policy = Policy.GetPolicy(contentType);
                }

                PolicyItem retentionPolicy = policy.Items[Expiration.PolicyId];
                if (retentionPolicy == null)
                {
                    policy.Items.Add(Expiration.PolicyId, RetentionFormulaXml);
                }
                else
                {
                    retentionPolicy.CustomData = RetentionFormulaXml;
                    retentionPolicy.Update();
                }
                policy.Name = PolicyName;
                policy.Description = PolicyDescription;
                policy.Statement = PolicyStatement;
                policy.Update();
            }
            catch (Exception x)
            {
                exception = x;
                return false;
            }
            return true;
        }

        // Configures an information policy for a document library.
        public bool Configure(SPDocumentLibrary library, out Exception exception)
        {
            exception = null;
            return true;
        }

        // Configures an information policy for a library folder.
        public bool Configure(SPFolder folder, out Exception exception)
        {
            exception = null;
            return true;
        }
    }
}
