﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.Office.RecordsManagement;
using Microsoft.Office.RecordsManagement.RecordsRepository;

namespace SharePointArchitects.FilePlanner.Configuration
{
    /// <summary>
    /// Defines a set of methods and properties that can be used to
    /// configure a SharePoint records center site.  Derived classes can
    /// apply different algorithms for configuring a given site.
    /// </summary>
    public class RecordsCenterConfiguration
    {
        FilePlan FilePlan = null;
        FilePlanSettings Settings = null;
        List<ContentTypeDescriptor> _contentTypes=null;
        List<DocumentLibraryDescriptor> _documentLibraries = null;
        List<InformationPolicyDescriptor> _informationPolicies = null;
        List<RoutingRuleDescriptor> _routingRules = null;

        // Default constructor to store the file plan and configuration options.
        public RecordsCenterConfiguration(FilePlan plan, FilePlanSettings settings)
        {
            FilePlan = plan;
            Settings = settings;
        }

        // Generic exception handler.
        protected virtual void HandleException(Exception x)
        {
            throw new Exception(
                string.Format("Records Center configuration failed using '{0}'", GetType().Name),
                x);
        }

        // Default strategy creates a content type for every record
        // series identified in the file plan.
        public virtual List<ContentTypeDescriptor> ContentTypes
        {
            get
            {
                if (_contentTypes == null)
                {
                    _contentTypes = new List<ContentTypeDescriptor>();

                    // Iterate through the file plan to create the content type descriptors.
                    foreach (RecordSpecification record in FilePlan.Records)
                        _contentTypes.Add(new ContentTypeDescriptor(FilePlan, record, Settings.RecordGrouping));
                }
                return _contentTypes;
            }
        }

        // Default strategy creates a separate document library for each
        // record series category, with subfolders for each subcategory.
        // Document libraries are pre-configured to accept the content types
        // that are associated with the records types that will be routed
        // to each library.
        public virtual List<DocumentLibraryDescriptor> DocumentLibraries
        {
            get
            {
                if (_documentLibraries == null)
                {
                    Dictionary<string, DocumentLibraryDescriptor> libs 
                        = new Dictionary<string, DocumentLibraryDescriptor>();

                    // Iterate through the file plan to construct a list of
                    // category names, using the path separator to identify
                    // folders.
                    foreach (RecordSpecification record in FilePlan.Records)
                    {
                        // Create a document library for each root category,
                        // then pass the remaining categories to the library to
                        // create it's folder hierarchy.
                        if (!string.IsNullOrEmpty(record.Classification))
                        {
                            string[] categories = record.Classification.Split('/');
                            if (!libs.ContainsKey(categories[0]))
                            {
                                // haven't seen this root category yet
                                libs[categories[0]] = new DocumentLibraryDescriptor(FilePlan, record.ContentType, categories);
                            }
                        }
                    }

                    _documentLibraries = libs.Values.ToList();
                }
                return _documentLibraries;
            }
        }

        // Default strategy creates a local information policy for each content type
        // that will be created.  Information policies are not shared among different
        // types and are not created for document libraries or folders.
        public virtual List<InformationPolicyDescriptor> InformationPolicies
        {
            get
            {
                if (_informationPolicies == null)
                {
                    // Return an empty list because local policies will be created
                    // for each content type using the default strategy.
                    _informationPolicies = new List<InformationPolicyDescriptor>();
                }
                return _informationPolicies;
            }
        }

        // Default strategy creates a routing rule for each content type that will
        // be created.  The routing rules are used primarily to move documents into
        // the appropriate document library or folder.
        public virtual List<RoutingRuleDescriptor> RoutingRules
        {
            get
            {
                if (_routingRules == null)
                {
                    _routingRules = new List<RoutingRuleDescriptor>();

                    // Create a routing rule for each document library and folder
                    // based on the content type associated with each record.
                    List<ContentTypeDescriptor> contentTypes = new List<ContentTypeDescriptor>();

                    foreach (RecordSpecification record in FilePlan.Records)
                        if (record.ContentType != null && !contentTypes.Contains(record.ContentType))
                            contentTypes.Add(record.ContentType);

                    // TODO: Specify conditions when creating routing rules.
                    // Conditions are derived from the context...
                    foreach (ContentTypeDescriptor contentType in contentTypes)
                        foreach (SharePointObjectDescriptor target in contentType.RoutingTargets)
                            _routingRules.Add(new RoutingRuleDescriptor(contentType, target));
                }
                return _routingRules;
            }
        }

        // Applies the configuration algorithm to a given records center site.
        public virtual bool Configure(SPSite recordsCenter, EcmDocumentRoutingWeb routingWeb, out Exception exception)
        {
            exception = null;
            try
            {
                // First, get the shared information policies.  These are site collection
                // policies that do not depend on content type or document library specific
                // metadata fields.
                foreach (InformationPolicyDescriptor policy in InformationPolicies)
                    if (!policy.Configure(recordsCenter, out exception))
                        return false;

                // Next, configure the content types.  These must be created before any
                // document libraries, but after any site collection policies that might
                // be associated with them.
                foreach (ContentTypeDescriptor contentType in ContentTypes)
                    if (!contentType.Configure(recordsCenter, out exception))
                        return false;

                // Next, configure the document libraries.  These may contain references to
                // content types and/or site collection policies.
                foreach (DocumentLibraryDescriptor library in DocumentLibraries)
                    if (!library.Configure(recordsCenter, out exception))
                        return false;

                // Finally, configure the routing rules.
                foreach (RoutingRuleDescriptor rule in RoutingRules)
                    if (!rule.Configure(recordsCenter, out exception))
                        return false;
            }
            catch (Exception x)
            {
                HandleException(x);
            }
            return true;
        }
    }
}
